// Not actually an AJAX request, but contains appropriate wrappers so
// that it imitates one nicely.
//
// Put a form like this in the document:
// <form><input type="file" name="Foo" /></form>
// Then call AFUuploadFile() with a specific request location, and a
// jQuery wrapper of the form object.
// To add additional parameters to the request, put hidden input tags inside the form.
//
// OnBegin will be called with Form as an argument when the upload begins. This is a good
// place to disable/remove the file input, because if you do it before calling AFUuploadFile,
// the file won't be included in the post request.
// OnComplete will be called with the response object and Form when the upload completes.
//
function AFUuploadFile(AFUlocation, Form, OnBegin, OnComplete, Params){
 var Guid = ("uploader" + (new Date()).getTime());
 
 Form.find('input[name="AFUlocation"]').remove();
 Form.append('<input type="hidden" name="AFUlocation" value="' + AFUlocation + '" />');

 var Iframe = $('<iframe name="' + Guid + '" src="about:blank" />');
 Iframe.css('display', 'none');
  
 $('body').append(Iframe);
 
 function dialogReload(){
  $(this).dialog('close');
  location.reload();
 }
 
 function onError(ErrorMessage){
  if(window.MiniLogin) {
   MiniLogin.show('File upload failed. ' + ErrorMessage, function(){ location.reload(); });
  } else {
   $('<div>' + ErrorMessage + '</div>').appendTo('body').dialog({
      modal:true, title:'Communication Error', buttons:{'Reload':dialogReload}});
  }
 }

 Iframe.load(function(EventObj){
  var Doc = null;
  if (Iframe.contentDocument)    Doc = Iframe.contentDocument;
  else if (Iframe.contentWindow) Doc = Iframe.contentWindow.document;
  else                           Doc = window.frames[Guid].document;
  var BodyHtml = $(Doc).find('body').html();
  
  if (!BodyHtml){
   onError('File upload failed. The server is not responding.');
  } else if (BodyHtml == 'Not logged in.'){
   onError('File upload failed. Your session has expired.');
  } else {
   var ResponseObject = null;
   if (JSON.parse){
    ResponseObject = JSON.parse(BodyHtml);
   }else{
    ResponseObject = eval('(' + BodyHtml + ')');
   }
  }
   
  // Small delay in removing iframe is to get around a bug in Firefox
  // that says the page is "Loading..." forever.
  setTimeout(function(){
    Iframe.remove();
    if (OnComplete && ResponseObject) OnComplete(ResponseObject, Params);
  }, 100);
 });

 Form
  .attr('action', '/ajax_file_upload')
  .attr('method', 'post')
  .attr('enctype', 'multipart/form-data')
  .attr('encoding', 'multipart/form-data')
  .attr('target', Guid);

 Form.get(0).submit();
 if (OnBegin) OnBegin(Form);
}
