var NISeditor=null;
var NISminAnnotationBlockWidth = 10;

function NIScreateIframe(iframeName, width, height, Parent) {
   var iframe;
   if (document.createElement && (iframe = document.createElement('iframe'))) {
      $(iframe).hide();
      iframe.name = iframe.id = iframeName;
      iframe.width = width;
      iframe.height = height + 4; // Add some padding for the editbox
      iframe.src = 'about:blank';
      iframe.style.padding = '0';
      iframe.style.border = '0';
      iframe.style.margin = '0';
      iframe.style.position = 'absolute';
      $(iframe).css("position", "absolute").css ("top", height + "px").css ("left", "0px");
      Parent.appendChild(iframe);
   }
   return iframe;
}

function NISblockLeftMargin(Line){
 var Count = 0;
 for(var i = 0; i < Line.length; ++i){
  if(Line.charAt(i) == " "){
   ++Count;
  }
  else{
   break;
  }
 }
 return Count;
}

function NISretrieveCodeBlockData(Editor){
 Editor.ifware.BlockDataSet = {};
 Editor.ifware.BlockDataSet.length = 0;

 var LineHandle = Editor.firstLine();
 var i = 0;
 var LineNumber = 1;
 var IsMultiLineComment = false;
 do {
  var Line = Editor.lineContent(LineHandle); 
   
  if(!Line.match(/^\s*$/)){
   var BlockData = {};
   BlockData.c = NISblockLeftMargin(Line);
   BlockData.w = Editor.ifware.CharWidth * Line.length; 
   BlockData.s = LineNumber;
   BlockData.e = BlockData.s + 1; 

   if(IsMultiLineComment){
    if(Line.match(/^.*\]\].*$/)){
     BlockData.type = "comment"; 
     IsMultiLineComment = false;  
    }
    else{
     BlockData.type = "comment"; 
    }  
   }
   else{  
    if(Line.match(/^\s*\-\-\[\[.*/)){
     BlockData.type = "comment"; 
     IsMultiLineComment = true;
    }
    else if(Line.match(/^\s*\-\-.*$/) || Line.match(/^\s*\-\-\[\[.*\]\].*/)){
     BlockData.type = "comment";
    }
    else if(Line.match(/^\s*require\s*.*$/)){
     BlockData.type = "requires";
    }
    else if(Line.match(/^function .*$/)){
     BlockData.type = "function";
    }
    else if(Line.match(/^end.*$/)){
     BlockData.type = "end";
    }
    else {
     BlockData.type = "line";
    }
   }
 
   Editor.ifware.BlockDataSet[i] = BlockData;  
   ++(Editor.ifware.BlockDataSet.length);
   ++i;
  }   
  ++LineNumber;
 }while(LineHandle = Editor.nextLine(LineHandle));
}

function NISretrieveMergeConflictData(Editor) {
   var LineHandle = Editor.firstLine();
   var LineNumber = 1;
   var BeginMergeConflictLineNumber = -1;
   var ConflictWidth = -1;

   do {
      var Line = Editor.lineContent(LineHandle);
      var BlockData = {};

      if (Line.match(/^.*BEGIN MERGE CONFLICT.*$/)) {
         BeginMergeConflictLineNumber = LineNumber;
      } else if (Line.match(/^.*END MERGE CONFLICT.*$/)) {
         BlockData.c = 0;
         BlockData.w = Editor.ifware.CharWidth * ConflictWidth;
         BlockData.s = BeginMergeConflictLineNumber;
         BlockData.e = LineNumber;
         BlockData.type = "merge_conflict";
         BlockData.t = "Merge Conflict";

         Editor.ifware.BlockDataSet[Editor.ifware.BlockDataSet.length++] = BlockData;
         ConflictWidth = -1;
         BeginMergeConflictLineNumber = -1;
      }
      
      if (BeginMergeConflictLineNumber != -1) {
         if (ConflictWidth < Line.length) {
            ConflictWidth = Line.length;
         }   
      }
      
      ++LineNumber;

   } while (LineHandle = Editor.nextLine(LineHandle));
}


function NIShideAnnotationsOnly(BlockData){
 if("annotation" == BlockData.type){
  return false;
 }
 return true;
}

function NISmaxCodeOfLineLength(BlockDataSet, CharWidth){
 var NumberOfBlocks = BlockDataSet.length;
 var MaxSoFar = 0; 
 for(var i = 0; i < NumberOfBlocks; ++i){
  if("annotation" == BlockDataSet[i].type){
   continue;
  } 
  var BlockData = BlockDataSet[i];
  var CurrentLineOfCodeLength = BlockData.c + BlockData.w / CharWidth + NISminAnnotationBlockWidth;
  if(CurrentLineOfCodeLength > MaxSoFar){
   MaxSoFar = CurrentLineOfCodeLength;
  }
 }
 return MaxSoFar;
}

function NISoptimalBlockWidth(BlockData, MaxLineOfCodeLength, ContainerWidth, CharWidth){
 var BlockWidth = Math.ceil(BlockData.w / CharWidth);
 if(ContainerWidth < MaxLineOfCodeLength){
  if(BlockData.c == MaxLineOfCodeLength){
   BlockWidth = NISminAnnotationBlockWidth; 
  }
  else if(BlockData.c + BlockWidth > MaxLineOfCodeLength + NISminAnnotationBlockWidth){
   BlockWidth = MaxLineOfCodeLength + NISminAnnotationBlockWidth - BlockData.c;
  }
 }
 else {
  if(BlockData.c == MaxLineOfCodeLength){
   BlockWidth = ContainerWidth - BlockData.c;
  }
  else if(BlockData.c + BlockWidth > ContainerWidth){
   BlockWidth = ContainerWidth - BlockData.c;
  }
 }
 return BlockWidth;  
}

function NISretrieveBlockStyle(Type){
 var Style = null;
 if("annotation" == Type){
  Style = {"class": "ANNblock"};   
 }
 else if("line" == Type){
  Style = {fill: "grey"}; 
 }
 else if("comment" == Type){
  Style = {"class": "lua-comment"};   
 }
 else if("requires" == Type){
  Style = {"class": "lua-stdfunc"};   
 }
 else if("function" == Type){
  Style = {"class": "lua-keyword"};   
 }
 else if ("end" == Type) {
    Style = { "class": "lua-keyword" };
 }
 else if ("merge_conflict" == Type) {
    Style = { "class": "merge-conflict" };
 }
 else {
  Style = {"class": "lua-token"}; 
 }
 return Style;
}

function NISonEditorInit(Editor){
   Editor.win.LOG.activate(true);
   var PopupDiv = $(Editor.ifware.Parent).parent().append('<div class="intellisensePopup" style="display:none;"></div>')
                          .find('.intellisensePopup')[0];
   ITSinitIntellisense(Editor, PopupDiv, Editor.ifware.IntellisenseStatusContainer);
   Editor.ifware.CharWidth = NIScharWidth(Editor);

   // Next line creates closure to annotate the code
   // We have to wait until the editor is initialized for this code to work - at least for Firefox
   NISannotateCode(Editor);
   ETHscrollerControl.initializeScroller(function(event){ EDTscrollEditor(event, Editor); });   
   $(".CodeMirror-iframe").get(0).contentWindow.onscroll = function(){ ETHscrollerControl.scroll(Editor) };

   if("true" != COOKIEread("thumbnail_show")){
    $("div#EDTthumbnailScroll").hide();
    $("div.CodeMirror-wrapping").css("right", "0px");
    $("#thumbnail_toggle").removeClass("on");
    COOKIEcreate("thumbnail_show", "false", 365);
   }

   // Bring up the import dialog if the source data is empty, or all whitespace
   var Code = Editor.getCode();
   if (!Code || Code.match(/^\s*$/)){
       SCMimportProjectAndData(Editor);
   }
}

function NIScreateEditorWithContent(Element, ScriptInfo, SampleDataIndex){
 var Editor =  NISeditor = new CodeMirror(Element, {
          height: "100%",
          parserfile: ["../../log/log.js", "../contrib/lua/js/parselua.js"],
          stylesheet: ["/js/mapper/ANNannotation2.css","/js/mapper/annotation.css", "/js/mapper/node_types.css"],
          lineNumbers: true,
          textWrapping: false,
          initCallback: NISonEditorInit,
          indentUnit : 3, 
	  content : ScriptInfo.ScriptData,
          path: "/js/cm91/js/",
          undoDelay: 350,
          noScriptCaching: true,
          iframeClass: "CodeMirror-iframe"
        });
 $(".CodeMirror-iframe").attr("style", "");
 Editor.options.saveFunction = function() { NISsave(Editor); }
 Editor.ifware = {};
 Editor.ifware.ShowEmptyNodes = false;
 NISsetEmptyTreenodeDisplayStyle(Editor, Editor.ifware.ShowEmptyNodes);
 Editor.ifware.Parent = Element;
 Editor.ifware.MainLuaFile = ScriptInfo.ScriptName;
 NISsetScriptName(Editor, ScriptInfo.ScriptName);
 Editor.ifware.ChannelGuid = ScriptInfo.ChannelGuid;
 Editor.ifware.UsesParameter = ScriptInfo.UsesParameter;
 if (ScriptInfo.UsesParameter){
  NISsetSampleData(Editor, ScriptInfo.CountOfSample, SampleDataIndex);
 }
 Editor.ifware.ScrollTops = {};
 PRJinitProjectManager(Editor, $('#EDTprojectManager'));

 return Editor;
}


var NISresizeDelay = (function(){
  var TimerId = 0;
  return function(Handler, Ms, Object){
    clearTimeout (TimerId);
    timer = setTimeout(Handler, Ms, Object);
  };
 })();

function NISdoResize(Editor){
 Editor.ifware.CharWidth = NIScharWidth(Editor);
 NISrefreshBlockInfo2(Editor, Editor.ifware.AnnotationBlockDataSet);
 ETHscrollerControl.resize(Editor);
}
 
// Create editor asynchrously loading content from the server.
function NIScreateEditor(Element, Callback, Params){
  Ajax.post('/init_editor', {'ChannelGuid':PAGgetField(Params,'ChannelGuid')},
  function(Response, TextStatus, RequestObject){
   console.log(Response);
   if (Response.ErrorMessage){
    MPRapp.renderError(Response.ErrorMessage);
    return;
   }
   var Editor = NIScreateEditorWithContent(Element, Response, Params.Id);
   ETHscrollerControl.show(Editor); 
   window.onresize = function(event){ 
    NISresizeDelay(NISdoResize, 500, Editor); 
   }
   
   if (typeof Callback != 'undefined'){
    Callback(Editor, Params);
   }
  }, 'json');
}

// Internet explorer is off by 5 pixels when showing anything annotation related.
function NISinternetExplorerOffset () {
   return (($.browser.msie) ? 5 : 0);
}

function NISgetEditorDiv(Editor){
   return NISgetEditorBody(Editor).childNodes[0];
}

function NISgetFrameDocument(Frame) {
   var Doc = null;
   if (Frame.contentDocument) {
      Doc = Frame.contentDocument;
   } else if (Frame.contentWindow) {
      // Internet Explorer
      Doc = Frame.contentWindow.document;
   } else if (Frame.document) {
      Doc = Frame.document;
   }
   return Doc;
}

function NISgetEditorBody(Editor) {
   var Frame = Editor.frame;
   return NISgetFrameDocument (Frame).body;
}

// TODO - this code is quite horrible and probably slow should really figure out if we really need it! Eliot
function NISgetLines(EditorDiv){
   var Lines = [];
   if (EditorDiv.childNodes.length === 0){ 
      console.log("Empty document");
      return [];
   }  // empty document
   var CurrentNode = EditorDiv.childNodes[0];
   var LineCount = 1;	
   do {
      if (CurrentNode.nodeName === "BR" || CurrentNode.nodeName === "br") { 
         LineCount++; 
      } else {
	 // little bit of overhead in multiple assignments here?  Not sure if it is worth optimizing. Eliot
	 Lines[LineCount] = CurrentNode.offsetTop;
      }
      if ($.browser.msie) {
	 CurrentNode = CurrentNode.nextSibling;
      } else {
	 CurrentNode = CurrentNode.nextElementSibling;
      }  
   } 
   while (CurrentNode);
   //console.log("Lines:", Lines);
   //console.log("Scanned ", Lines.length, "lines");
   // The above routine will leave us with some holes in our array in some browsers - we need to find the typical offset between tlines.
   // Incidentally the reason we have this routine is that Chrome on Mac OS X, on Eliot's machine has some lines of 16 pixels and some of 17
   var FirstNonBlank;
   for (FirstNonBlank=1; FirstNonBlank < Lines.length && (typeof Lines[FirstNonBlank] == 'undefined' || Lines[FirstNonBlank] == 0); FirstNonBlank++);
   //console.log("FirstNonBlank = ", FirstNonBlank);
   var Offset = (Lines[FirstNonBlank]) / (FirstNonBlank-1);
   //console.log("Offset = ", Offset);
   //console.log("Lines:", Lines);
   var i;
   for (i = FirstNonBlank-1; i > 0; i--){
      Lines[i] = Lines[i+1] - Offset;
   }
   for (i = FirstNonBlank+1; i < Lines.length; i++){
      if (typeof Lines[i] === 'undefined' || Lines[i] === 0){
         Lines[i] = Lines[i-1] + Offset; 
      }     
   }
   for (i=0; i < Lines.length; i++){
   //   console.log("Line:", i, Lines[i]);
      if (Lines[i] <= 0) Lines[i] = 0.5;
   }
   //console.log("Lines:", Lines);
   return Lines;
}

function NISlistLines(Editor) {
   var Lines = NISgetLines(NISgetEditorDiv(Editor));
   for (i=1; i < Lines.length; i++) {
      console.log('[' + i + '] @' +  Lines[i] + ' h=' + (Lines[i+1] - Lines[i]));
   }
}

// Pass this function a Code Mirror Editor - this is a diagnostic function with no production need.
function NISshowLines(Editor){
   var Lines = NISgetLines(NISgetEditorDiv(Editor));
   var SimpleOffset = Lines[2] - Lines[1];

   function divWithTop(Top, Left, Color){
      var Div = document.createElement('div');
      Div.style.position = 'absolute';
      Div.style.top = Top + 'px';
      Div.style.left = Left + 'px';
      Div.style.height = '1px';
      Div.style.width = '200px';
      Div.style.backgroundColor = Color;
      return Div;
   }

   NISclearAnnotations(Editor);

   var Body = NISgetEditorBody(Editor);

   for (var i = 1; i < Lines.length; i++) {
      var SimplePosition = Lines[1] + (i-1)* SimpleOffset;
      Body.appendChild(divWithTop(Lines[i], 0, 'red'));
      Body.appendChild(divWithTop(SimplePosition, 200, 'green'));
   }
}

function NISeditorIsEmpty(Editor) {
   var Body = NISgetEditorBody(Editor);
   return (Body.childNodes[0].childNodes.length === 0);
}

function NIScharWidth(Editor) {
   var Body = NISgetEditorBody(Editor);
   console.assert(Body.childNodes.length > 0);
   console.assert(!NISeditorIsEmpty(Editor));
   var Element = Body.childNodes[0].childNodes[0];
   var TestSpan = $("<span id='width_test'>Test</span>");
   $(Body.childNodes[0]).append(TestSpan);
   var CharWidth = TestSpan.get(0).offsetWidth / 4;
   $(Body.childNodes[0]).find("#width_test").remove();

   return CharWidth; 
}

function NIScharHeight(Editor) {
   var Body = NISgetEditorBody(Editor);
   console.assert(Body.childNodes.length > 0);
   console.assert(!NISeditorIsEmpty(Editor));
   var Element = Body.childNodes[0].childNodes[0];
   var TestSpan = $("<span id='height_test'>Test</span>");
   $(Body.childNodes[0]).append(TestSpan);
   var CharHeight = TestSpan.get(0).offsetHeight;
   $(Body.childNodes[0]).find("#height_test").remove();

   return CharHeight; 
}

function NISclearAnnotations(Editor) {
   var Body = NISgetEditorBody(Editor);
   console.log("NISclearAnnotations:", Body, Body.childNodes);
   for (var i=Body.childNodes.length-1; i > 0; i--) {
      var Child = Body.childNodes[i];
      if (Child.id != 'code-mirror-inner-code') Body.removeChild(Child);
   }
}

function NISsetButtonVisibility(Elements, Hidden) {
   Images = Elements.find("img");
   Images.each(function() {
      var FileName = $(this).attr("src");
      var StartIndex = (FileName.indexOf('/') >= 0) ? FileName.lastIndexOf('/') : 0;
      var DesiredPrefix = Hidden ? "dis" : "ena";
      if (FileName.substr(DesiredPrefix.length, 3) != DesiredPrefix) {
         var NewFileName = FileName.substr(0, StartIndex + 1) + DesiredPrefix + FileName.slice(StartIndex + DesiredPrefix.length + 1, FileName.length);
         $(this).attr("src", NewFileName);
      }
   }
   );
}

// TODO - perhaps by using an orgthonal class for these elements this code could be simplified
function NISclearErrors(Editor){
   $(NISgetEditorBody(Editor)).find(".error-close,.error-box,.error-highlight-row").remove();
}

function NISclearErrorBox(Editor){
   $(NISgetEditorBody(Editor)).find('.error-box').remove();
}

function NISgetLineNumberDiv(Editor, LineNumber){
   return Editor.lineNumbers.firstChild.childNodes[LineNumber-1];
}

function NISmarkLineAsDirty(Editor, LineNumber){
 Editor.ifware.CurrentDirtyLine = LineNumber;
 var LineNumberDiv = NISgetLineNumberDiv(Editor, LineNumber);
 $(LineNumberDiv).addClass('dirty');
 var LineTops = NISgetLines(NISgetEditorDiv(Editor))
 var LineTop = LineTops[LineNumber];
 NISmarkAnnotationAsStale(Editor, LineNumber, LineTop);
}

function NISmarkLineAsClean(Editor, LineNumber){
 var LineNumberDiv = NISgetLineNumberDiv(Editor, LineNumber);
 $(LineNumberDiv).removeClass('dirty');
}

function NISprepareLuaPostInfo(Editor, PostVariables, NeedCode){
 if (NeedCode){
  PostVariables.Code = Editor.getCode();
 }
 PostVariables.LuaFileName = Editor.ifware.LuaFileName;
 PostVariables.ChannelGuid = Editor.ifware.ChannelGuid;
 PostVariables.SampleDataIndex = Editor.ifware.CurrentSampleData;
}

function NISsetSampleData(Editor, SampleDataCount, SampleDataIndex){
 console.log("Sample data index = ", SampleDataIndex);
 Editor.ifware.CountOfSampleData = SampleDataCount;
 if (!SampleDataIndex) SampleDataIndex = COOKIEread(Editor.ifware.ChannelGuid + '_EditorSampleDataIndex');
 if (SampleDataIndex) {
  SampleDataIndex = parseInt(SampleDataIndex);
  if (SampleDataIndex > 0 && SampleDataIndex <= SampleDataCount){
   Editor.ifware.CurrentSampleData = SampleDataIndex;
   console.log("Editor.ifware.CurrentSampleData = ", Editor.ifware.CurrentSampleData);
   return;
  }
 }
 Editor.ifware.CurrentSampleData = 1;
}

function NISlogWithTime(Line){
 var Now = new Date();
 console.log(Now.getSeconds(), Now.getMilliseconds(), Line);
}

function NISupdateAnnotations(Editor, NeedCode, OldFile){
 NISlogWithTime('NISupdateAnnotations');
 NISclearErrors(Editor);
 var Values = {};
 NISprepareLuaPostInfo(Editor,Values, NeedCode);  // Use common helper function to pack in Lua information
 if (Editor.ifware.DisableAnnotations){
  Editor.ifware.AnnotationsStatusContainer.innerHTML = 'Disabled';
  NISclearAnnotations(Editor);
  Values.Disabled = true;
 }
 if (Editor.ifware.CountOfSampleData == 0){
  Editor.ifware.AnnotationsStatusContainer.innerHTML = 'No data to generate annotations.';
  NISclearAnnotations(Editor);
  Values.Disabled = true;
 }
 Values.File = NISgetScriptName(Editor);
 if (OldFile){
  Values.OldFile = OldFile;
 }
 NISlogWithTime('Sending annotation request.');
 console.log('Values',Values);
 Ajax.post( '/annotation_info2', Values, function(R, T, RO) {NISprocessAnnotation2(R,T,RO, Editor); }, 'json');
 Editor.ifware.AnnotationsStatusContainer.innerHTML = '<img src="/js/mapper/images/spinner.gif" />';
}

function NISsetScriptName(Editor, ScriptName){
 console.log('Script:', ScriptName);
 Editor.ifware.LuaFileName = ScriptName;
}

function NISonFileChanged(Editor, NewFile){
 console.log("File changed:", NewFile);
 // Store scrollTop of current file.
 if (Editor.ifware.LuaFileName && NewFile != Editor.ifware.LuaFileName){
  var ScrollTop = $(NISgetEditorBody(Editor)).scrollTop();
  Editor.ifware.ScrollTops[Editor.ifware.LuaFileName] = ScrollTop;
 }
 var OldFile = NISgetScriptName(Editor);
 NISsetScriptName(Editor, NewFile);
 NISupdateAnnotations(Editor, Editor.ifware.CurrentDirtyLine ? true : false, OldFile);
}

function NISgetScriptName(Editor){
 return Editor.ifware.LuaFileName;
}

function NISsetCode(Editor, Code){
 Editor.ifware.ChangingFile = true;
 try {
  Editor.setCode(Code);
 } finally {
  Editor.ifware.ChangingFile = undefined;
 }
 // Restore scrollTop for new file, or set to 0 if it hasn't been opened previously.
 NISgetEditorBody(Editor).scrollTop = Editor.ifware.ScrollTops[Editor.ifware.LuaFileName] || 0;
}

// If the save was successful, Callback will be called with Editor as the argument.
function NISsave(Editor, Callback){
 if (Editor.ifware.CurrentDirtyLine){
  NIScleanDirtyLine(Editor);
  NISupdateAnnotations(Editor, true);
 }
 Ajax.post('/annotation_save_program', { 'ChannelGuid' : Editor.ifware.ChannelGuid },
            function(Response, TextStatus, RequestObject){
   if (Response.ErrorMessage) {
    Editor.ifware.AnnotationsStatusContainer.innerHTML = Response.ErrorMessage;  // TODO switch to single status container
    return;
   }
   else if (Callback){
    Callback(Editor);
   }
  },'json');
}

function NIScleanDirtyLine(Editor){
   NISmarkLineAsClean(Editor, Editor.ifware.CurrentDirtyLine);
   Editor.ifware.CurrentDirtyLine = null;
}

function NISisWhitespace(Text){
   return Text.match(/^\s*$/) != null;
}

// Enclosure - has state
function NISannotateCode(iEditor){
   // We set up out 'member variables' for the closure kind of like an object.
   var m_Doc = iEditor.win.document.body;
   var m_Editor = iEditor;
   
   if (typeof (m_Editor.ifware) === "undefined") m_Editor.ifware = {};
   
   m_Editor.ifware.CurrentDirtyLine = null;
   m_Editor.ifware.BlockInfo = {};
   
   function onCursorActivity(CursorSpan){
      var CursorLineNumber = m_Editor.lineNumber(m_Editor.cursorLine());
      if (m_Editor.ifware.CurrentDirtyLine && CursorLineNumber != m_Editor.ifware.CurrentDirtyLine){
         NIScleanDirtyLine(m_Editor);
         NISupdateAnnotations(m_Editor, true);
      }
   }

   // Setup cursor activity callback
   if (m_Editor.options.cursorActivity){
      var OriginalCursorActivityCallback = m_Editor.options.cursorActivity;
      m_Editor.options.cursorActivity = function(CursorSpan){
         onCursorActivity(CursorSpan);
         OriginalCursorActivityCallback(CursorSpan);
      }
   }
   else{
      m_Editor.options.cursorActivity = onCursorActivity;
   }


   console.assert(m_Doc);
   
   iEditor.editor.history.onChange = function(){
      if (m_Editor.ifware.CurrentDirtyLine) {
         NIScleanDirtyLine(m_Editor);
      }
      
      NISclearErrorBox(m_Editor);
      
      if (m_Editor.ifware.ChangingFile) {
       // The entire file is changing - we don't need to handle this event.
       return;
      }
      
      var ChangeHistory = m_Editor.editor.history.history;
      if (ChangeHistory.length) {
         // TODO - not sure why the codemirror implementation has the history wrapped up inside two arrays ([0][0]).
         var MostRecentChange = ChangeHistory[ChangeHistory.length-1][0][0];
         var CursorLine = m_Editor.cursorLine();
         var CursorLineNumber = m_Editor.lineNumber(CursorLine);
         
         if (!MostRecentChange.to){
           console.log('MostRecentChange.to is not a valid line.');
           return;  
         }
         
         // If the current line is whitespace then we won't consider the line dirty.
         // Otherwise, if the current line is the recently changed line, we'll consider it dirty.
         // If 'to' does not have a parent node, it is no longer in the document.
         var MarkCurrentLineAsDirty = !NISisWhitespace(m_Editor.lineContent(CursorLine)) && (
            (MostRecentChange.to.parentNode && CursorLineNumber == m_Editor.lineNumber(MostRecentChange.to) - 1) ||
            (!MostRecentChange.to.parentNode && CursorLineNumber == m_Editor.lineNumber(MostRecentChange.from))
         );
         
         if (MarkCurrentLineAsDirty) {
            NISmarkLineAsDirty(m_Editor, CursorLineNumber);
            return;
         }
      }
      NISretrieveCodeBlockData(m_Editor);
      NISretrieveMergeConflictData(m_Editor);
      NISupdateAnnotations(m_Editor, true);
   }
   
   NISupdateAnnotations(m_Editor, false);
}

function NISsetEmptyTreenodeDisplayStyle(Editor, NewDisplayValue) {
 // We want "EditorDocument" to be the document in which the treeviews will be displayed.
 var EditorDocument = document;
 for (var i = 0; i < EditorDocument.styleSheets.length; i++){
  var StyleSheet = EditorDocument.styleSheets[i];
  if (StyleSheet.href && StyleSheet.href.indexOf('node_treeview.css') != -1){
   var Rules = StyleSheet.cssRules ? StyleSheet.cssRules : StyleSheet.rules;
   Rules[0].style.display = NewDisplayValue;
   return;
  }
 }
}

function NISchangeSampleData(Editor, NewIndex){
 console.log("NISchangeSampleData = ", NewIndex)
 $('div.EDTsd_goto').remove();
 Editor.ifware.CurrentSampleData = NewIndex;
 NISupdateSampleDataControl(Editor);
 COOKIEcreate(Editor.ifware.ChannelGuid + '_EditorSampleDataIndex', NewIndex, 60);
 NISupdateAnnotations(Editor, false);
}

function NISinitSampleDataButton(Button, DisabledCondition, MoveTo, Editor){
 Button.addClass(DisabledCondition ? 'disabled' : 'enabled');
 Button.click(function(Event){
  Event.preventDefault();
  if (!DisabledCondition){
   NISchangeSampleData(Editor, MoveTo);
  }
 });
}

function NISupdateSampleDataControl(Editor, Control){
 if (!Control){
  Control = $('#sample_data_control');
 }
 
 var CurrentSd = Editor.ifware.CurrentSampleData;
 var TotalSd = Editor.ifware.CountOfSampleData;
 var UsesParameter = Editor.ifware.UsesParameter;
 
 // TODO we make the assumption here that a sample message's Id will always be one higher
 // than its index.  This may not be true in the future, if we allow sample data to be
 // deleted.  (Eliot may have changed this)
 var SdParams = Params;
 SdParams.Id = CurrentSd;
 SdParams.TotalSd = TotalSd;
 Control.html(
   '<span class="label">' + (UsesParameter ? PAGlink('SampleData', Params, 'Sample Data') : 'Sample Data') + ': </span>' +
   '<a href="" class="button_icon connected" id="msg_first" title="First Message"><span>First</span></a>' +
   '<a href="" class="button_icon connected" id="msg_prev" title="Previous Message"><span>Previous</span></a>' +
   '<a href="" class="button_msg ' + (UsesParameter ? 'enabled' : 'disabled') + '" title="Go to Message"><span>' +
   (TotalSd ? (CurrentSd + ' of ' + TotalSd) : 'None') +
   '</span></a>' +
   '<a href="" class="button_icon connected" id="msg_next" title="Next Message"><span>Next</span></a>' +
   '<a href="" class="button_icon connected" id="msg_last" title="Last Message"><span>Last</span></a>' + 
   (TotalSd ? BTNtoolbar('EditSampleData', 'Edit Message', SdParams, 'off', 'edit_message', 'Edit Message') : ''))
  
 .find('a#msg_first').each(function(){
  NISinitSampleDataButton($(this), !UsesParameter || CurrentSd == 1, 1, Editor);
 }).end()
 
 .find('a#msg_prev').each(function(){
  NISinitSampleDataButton($(this), !UsesParameter || CurrentSd == 1, CurrentSd-1, Editor);
 }).end()
 
 .find('a.button_msg').click(function(Event){
  Event.preventDefault();
  if (!UsesParameter) return;
  var Div = $('div.EDTsd_goto');
  if (Div.size()){
   Div.remove();
  }else{
   Div = $('\
      <div id="mb-parent" class="EDTsd_goto">' +
         (TotalSd ? '<div id="goto-row">Go to <input class="EDTsd_gotoInput"><button id="goto-btn">Go</button></div>' : '') + '\
      </div>');
   $('body').append(Div);
   if(window.MiniBrowser) {
      MiniBrowser.create({
         'parent': $('#mb-parent'),
         'source': PAGgetField(Params, 'ChannelName'),
         'target': Editor.ifware.ChannelGuid,
         'onOpen': function() { $('#goto-row').hide(); },
         'onCancel': function() { Div.remove(); },
         'onStart': function() { Div.remove(); },
         'onSuccess': function(Index, Size) {
            Editor.ifware.CountOfSampleData += Size;
            NISchangeSampleData(Editor, Index); }
      });
   }
   var Button = $(this.firstChild);
   var ButtonOffset = Button.offset();
   Div.css('left', ButtonOffset.left - 1);
   Div.css('top',  ButtonOffset.top + Button.outerHeight() - 1);
   
   var Input = Div.find('#goto-row input');
   Input.val(Editor.ifware.CurrentSampleData);
   Input.focus();
   Input.select();
   
   var NISonGoto;
   if(!TotalSd) NISonGoto = function() { Div.remove(); };
   else NISonGoto = function(EventObj) {
    var NewSd = parseInt(Input.val());
    if (isNaN(NewSd) || NewSd < 1){
     NewSd = 1;
    }else if (NewSd > Editor.ifware.CountOfSampleData){
     NewSd = Editor.ifware.CountOfSampleData;
    }
    NISchangeSampleData(Editor, NewSd);
   };
  
   Input.keydown(function(EventObj){
    if      (EventObj.keyCode == '13') NISonGoto(EventObj);
    else if (EventObj.keyCode == '27') Div.remove();
   });
   
   Div.find('#goto-btn').click(NISonGoto);
  }
 }).end()
 
 .find('a#msg_next').each(function(){
  NISinitSampleDataButton($(this), !UsesParameter || CurrentSd >= TotalSd, CurrentSd+1, Editor);
 }).end()
 
 .find('a#msg_last').each(function(){
  NISinitSampleDataButton($(this), !UsesParameter || CurrentSd >= TotalSd, TotalSd, Editor);
 });
}

function NISsetButtonEnabled(Button, Enabled){
 Button.addClass(Enabled ? 'enabled' : 'disabled');
 Button.removeClass(Enabled ? 'disabled' : 'enabled');
}

function NIScreateToolbar(Editor, Params)
{
 var ToolbarH = '<div class="section">' + '<span id="sample_data_control"></span></div>';
 
 ToolbarH +=
  '<div class="section">' +
   BTNtoolbar('', 'Save Milestone', {}, 'off', 'milestone', 'Save Milestone') + 
   BTNtoolbar('', 'View Saved Milestones', {}, 'off', 'revisions', 'View Saved Milestones') + 
  '</div>' + 
  '<div class="section">' +
   BTNtoolbar('', 'Annotations', {}, 'on', 'annotations', 'Show Annotations') + 
   BTNtoolbar('', 'Empty Nodes', {}, 'off', 'emptynodes', 'Show Empty Nodes') + 
  '</div>' +
  '<div class="section">' +
   BTNtoolbarWithJavascript('javascript:EDTtoggleProjectPane(this);', 'Project Manager', 'on', 'project_pane_toggle', 'Show Project Manager') + 
   BTNtoolbar('', 'Thumbnail', {}, (("true" == COOKIEread("thumbnail_show")) ? 'on' : 'off'), 'thumbnail_toggle', 'Show Thumbnail') + 
  '</div>';
  
 return $(document.createElement('form')).html(ToolbarH)
 .find("#sample_data_control").each(function(){
  NISupdateSampleDataControl(Editor, $(this));
 }).end()
 .find("a#annotations").click(function(Event){
  Event.preventDefault();
  var jThis = $(this);
  console.log("Annotations toggled.");
  if (jThis.hasClass('on')){
   jThis.removeClass('on');
   NISsetButtonEnabled($('a#emptynodes'), false);
   Editor.ifware.DisableAnnotations = true;
   ETHscrollerControl.setBlockFilter(function(BlockData){ return NIShideAnnotationsOnly(BlockData); });
  } else {
   jThis.addClass('on');
   NISsetButtonEnabled($('a#emptynodes'), true);
   Editor.ifware.DisableAnnotations = false;
   ETHscrollerControl.setBlockFilter(null);
  }
  console.log("About to update annotations.");
  NISupdateAnnotations(Editor, true); 
 }).end()
 .find("a#emptynodes").click(function(Event){
  Event.preventDefault();
  var jThis = $(this);
  if (jThis.hasClass('disabled')) return;
  if (jThis.hasClass('on')){
   jThis.removeClass('on');
   Editor.ifware.ShowEmptyNodes = false;
  } else {
   jThis.addClass('on');
   Editor.ifware.ShowEmptyNodes = true;
  }
  NISsetEmptyTreenodeDisplayStyle(Editor, Editor.ifware.ShowEmptyNodes ? '' : 'none');
 }).end()
 .find("a#revisions").click(function(Event) {
     Event.preventDefault();
     NISsave(NISeditor, SCMviewMilestonesDlg);
 }).end()
 .find("a#milestone").click(function(Event) {
     Event.preventDefault();
     NISsave(NISeditor, SCMsaveMilestoneDlg);
 }).end()
  .find("a#thumbnail_toggle").click(function(Event) {
     Event.preventDefault();
     EDTtoggleThumbnail(Editor);
 }).end()[0];
 
}

function NIScreateStatusBar(Editor, Container)
{
 Container.html('<div class="an_sb">Auto-Completion: <span id="intellisense_status"></span></div><div class="an_sb an_sb_right">Annotations: <span id="annotations_status"></span></div>')
  .find('#intellisense_status').each(function(){
   Editor.ifware.IntellisenseStatusContainer = this;
  }).end()
  .find('#annotations_status').each(function(){
   Editor.ifware.AnnotationsStatusContainer = this;
  }).end()[0];
}
