function ETHscroller(NewContainerId){
 EmbeddedScrollBarWidth = 5;
 GroupId = "EDTthumbnail_group";
 GroupHeight = 0;
 GroupWidth = 0;

 ContainerId = NewContainerId;

 function defaultBlockFilter(BlockData){
  return true;
 }

 function defaultStylizer(Type){
  return {fill: "black"};   
 }

 function defaultMaxLineLengthCalculator(BlockDataSet, CharWidth){
  var NumberOfBlocks = BlockDataSet.length;
  var MaxSoFar = 0; 
  for(var i = 0; i < NumberOfBlocks; ++i){
   var BlockData = BlockDataSet[i];
   var CurrentLineLength = BlockData.c + BlockData.w / CharWidth;
   if(CurrentLineLength > MaxSoFar){
    MaxSoFar = CurrentLineLength;
   }
  }
  return MaxSoFar;   
 }

 function defaultOptimalBlockWidthCalculator(BlockData, MaxLineLength, ContainerWidth, CharWidth){
  return Math.ceil(BlockData.w / CharWidth);
 }

 BlockFilter = defaultBlockFilter;
 Stylizer = defaultStylizer;
 MaxLineLengthCalculator = defaultMaxLineLengthCalculator;
 OptimalBlockWidthCalculator = defaultOptimalBlockWidthCalculator;

 function verticalScale(){
  var GroupTransformation = $("g#" + GroupId).attr("transform");
  var ScaleAttr = GroupTransformation.match(/scale\s*\(\s*\d(\.\d+)?\s*\,\s*\d(\.\d+)?\s*\)/g);
  var Scale = ScaleAttr[0].match(/\d(\.\d+)?/g);

  return parseFloat(Scale[1]);
 }

 function horizontalScale(){
  var GroupTransformation = $("g#" + GroupId).attr("transform");
  var ScaleAttr = GroupTransformation.match(/scale\s*\(\s*\d(\.\d+)?\s*\,\s*\d(\.\d+)?\s*\)/g);
  var Scale = ScaleAttr[0].match(/\d(\.\d+)?/g);

  return parseFloat(Scale[0]);
 }

 function showScrollBar(Width, Editor){
  var Container = $("#" + ContainerId);
  var svg = Container.svg("get");
  var g = svg.getElementById(GroupId);

  var EditorLineHeight = NIScharHeight(Editor);
  var ThumbnailScrollBarHeight = Math.min(GroupHeight, $("div.CodeMirror-wrapping").get(0).clientHeight / EditorLineHeight);
  var ScrollAdjustmentFactor = $(".CodeMirror-iframe").get(0).contentWindow.document.body.scrollHeight / parseInt($("svg").attr("height")) * verticalScale();

  var ThumbnailScrollBarY = $(".CodeMirror-iframe").get(0).contentWindow.scrollY / ScrollAdjustmentFactor;

  if(ThumbnailScrollBarY + ThumbnailScrollBarHeight > GroupHeight){
   ThumbnailScrollBarY = GroupHeight - ThumbnailScrollBarHeight;
  }

  svg.rect(g, 0, Math.max(1, ThumbnailScrollBarY), Width + 1, ThumbnailScrollBarHeight, {"class": "lua-scrollbar"});
 }

 function configureCanvas(Height, IsOverwriteConfig, Editor){
  var Container = $("#" + ContainerId);
  var svg = Container.svg("get");

  var EditorLineHeight = NIScharHeight(Editor);
  var ThumbnailScrollBarHeight = $("div.CodeMirror-wrapping").get(0).clientHeight / EditorLineHeight;

  svg.configure({viewbox: "0 0 " + Container.get(0).clientWidth + " " + Container.get(0).clientHeight, preserveAspectRatio: "none"}, IsOverwriteConfig); 

  if(Height + 1 < Container.get(0).clientHeight){
   svg.configure({height: Height + 1}, false);
  }
  else{
   svg.configure({height: Container.get(0).clientHeight}, false); 
  }
  svg.configure({width: Container.get(0).clientWidth}, false)
 }

 TimeoutId = -1;
 function showToolTip(Block){
  if(!Block.attr("tooltip")){
   return;
  }

  clearTimeout(TimeoutId);

  var Container = $("#" + ContainerId);

  TOOLtooltipLink(Block.attr("tooltip"), null, null, 0);
  TimeoutId = setTimeout("TOOLtooltipClose();", 2000);
 }

 function processScrollBarMouseMove(event){
  var Container = $("#" + ContainerId);

  var OffsetLeft = Container.offset().left;
  var OffsetTop = Container.offset().top;

  var X = (event.clientX - OffsetLeft) / horizontalScale(); 
  var Y = event.clientY - OffsetTop;

  $("rect").each( 
   function(i){
    if($(this).attr("tooltip")){
     var Left = parseInt($(this).attr("x"));
     var Top = parseInt($(this).attr("y"));
     var Right = Left + parseInt($(this).attr("width"));
     var Bottom = Top + parseInt($(this).attr("height"));

     if((X >= Left && X <= Right) && (Y >= Top && Y <= Bottom)){
      showToolTip($(this));
      return false;
     }
    }
   }
  );  
 }

 function transitionToScrollMode(){
  $("rect").unbind();
 }

 function transitionToStaticMode(){
  $("rect[tooltip]").bind("mouseenter", function(){ showToolTip($(this)); }); 
  $("rect.lua-scrollbar").unbind().bind("mousemove", function(event){ processScrollBarMouseMove(event); }); 
 }

 function drawControl(svg, Editor){
  var Container = $("#" + ContainerId);
  var g = svg.group(null, GroupId);

  GroupHeight = 0;
  GroupWidth = 0; 

  var ContainerWidth = Container.get(0).clientWidth;
  var ContainerHeight = Container.get(0).clientHeight;

  var MaxLineOfCodeLength = MaxLineLengthCalculator(Editor.ifware.BlockDataSet, Editor.ifware.CharWidth);

  var NumberOfBlocks = Editor.ifware.BlockDataSet.length;
  for(var i = 0; i < NumberOfBlocks; ++i){
   var BlockData = Editor.ifware.BlockDataSet[i];
   var Style = Stylizer(BlockData.type);

   var BlockWidth = OptimalBlockWidthCalculator(BlockData, MaxLineOfCodeLength, ContainerWidth, Editor.ifware.CharWidth);

   if(BlockFilter(BlockData)){
    svg.rect(g, EmbeddedScrollBarWidth + BlockData.c, BlockData.s, BlockWidth, BlockData.e - BlockData.s, Style); 
    if(BlockData.t){
     $("rect:last").attr("tooltip", BlockData.t);
    }
   }
   else{
    svg.rect(g, EmbeddedScrollBarWidth + BlockData.c, BlockData.s, BlockWidth, BlockData.e - BlockData.s, {fill: $("div#EDTthumbnailScroll svg").css("background-color")}); 
   }

   if(GroupHeight < BlockData.e){
    GroupHeight = BlockData.e;
   }
   if(GroupWidth < (BlockWidth + BlockData.c)){
    GroupWidth = BlockWidth + BlockData.c;
   }
  } 

  configureCanvas(GroupHeight, true, Editor);

  if(0 < GroupWidth && 0 < GroupHeight){
   $("g#" + GroupId).attr("transform", "scale(" + Math.min(1, ContainerWidth/GroupWidth) + "," + Math.min(1, ContainerHeight/GroupHeight) + ")");
   showScrollBar(Math.max(ContainerWidth, GroupWidth), Editor);
  }
  else{
   $("g#" + GroupId).attr("transform", "scale(1,1)");
  }
 
  transitionToStaticMode();
 }

 this.setBlockFilter = 
  function(Handler) {
   BlockFilter = Handler ? Handler : defaultBlockFilter;
  }

 this.setStylizer = 
  function(Handler){
   Stylizer = Handler ? Handler : defaultStylizer;
  }

 this.setMaxLineLengthCalculator = 
  function(Handler){
   MaxLineLengthCalculator = Handler ? Handler : defaultMaxLineLengthCalculator;
  }
 
 this.setOptimalBlockWidthCalculator = 
  function(Handler){
   OptimalBlockWidthCalculator = Handler ? Handler : defaultOptimalBlockWidthCalculator;
  }

 this.retrieveVerticalScale = 
  verticalScale;

 this.retrieveHorizontalScale = 
  horizontalScale;

 this.initializeScroller = 
  function (Handler){
   var Container = $("#" + ContainerId);

   Container.unbind("mousedown").bind("mousedown", Handler).bind("mousedown", function(){ $(this).unbind("mousemove").bind("mousemove", Handler); transitionToScrollMode(); });

   Container.unbind("mouseup").bind("mouseup", function(){ $(this).unbind("mousemove"); transitionToStaticMode(); });
   Container.unbind("mouseleave").bind("mouseleave", function(){ $(this).unbind("mousemove"); transitionToStaticMode(); });

   $("rect").unbind(); 
   $("g").unbind();
   $("svg").unbind();
  }

 this.scroll = 
  function (Editor){
   var svg = $("#" + ContainerId).svg("get");

   $("rect.lua-scrollbar").remove();
 
   showScrollBar(Math.max($("#" + ContainerId).get(0).clientWidth, GroupWidth), Editor);
  }

 this.showError = 
  function (ScriptError, LineLength){
   var Container = $("#" + ContainerId);
   var svg = Container.svg("get");
   var g = svg.getElementById(GroupId);

   var ErrorLines = ScriptError.Description.match(/.*\r/g);
   
   var Width = 0;
   var Height = 0;
   if(ErrorLines){
    Height = ErrorLines.length;
    for(var i = 0; i < Height; ++i){
     var CurrentWidth = ErrorLines[i].length;
     if(CurrentWidth > Width){
      Width = CurrentWidth;
     }
    }
   }
   else{
    Height = 1;
    Width = ScriptError.Description.length;
   }

   if(1 == Height){
    svg.rect(g, 1, ScriptError.LineNumber, Math.max(GroupWidth, Container.get(0).clientWidth), Height, {fill: "#F6CECE"});    
   }   

   svg.rect(g, LineLength + 5, ScriptError.LineNumber, Width, Height, {"class": "lua-error"});      
  }

 this.clear = 
  function (Editor){
   $("#" + ContainerId).svg("destroy");
  }

 this.resize = 
  function (Editor){
   var Container = $("#" + ContainerId);
   var svg = Container.svg("get");
   var g = svg.getElementById(GroupId);

   GroupHeight = 0;
   GroupWidth = 0; 

   var ContainerWidth = Container.get(0).clientWidth;
   var ContainerHeight = Container.get(0).clientHeight;

   var MaxLineOfCodeLength = MaxLineLengthCalculator(Editor.ifware.BlockDataSet, Editor.ifware.CharWidth);

   var NumberOfBlocks = Editor.ifware.BlockDataSet.length;
   for(var i = 0; i < NumberOfBlocks; ++i){
    var BlockData = Editor.ifware.BlockDataSet[i];

    var BlockWidth = OptimalBlockWidthCalculator(BlockData, MaxLineOfCodeLength, ContainerWidth, Editor.ifware.CharWidth);

    if(GroupHeight < BlockData.e){
     GroupHeight = BlockData.e;
    }
    if(GroupWidth < (BlockWidth + BlockData.c)){
     GroupWidth = BlockWidth + BlockData.c;
    }
   } 

   configureCanvas(GroupHeight, false, Editor);

   if(0 < GroupWidth && 0 < GroupHeight){
    $("g#" + GroupId).attr("transform", "scale(" + Math.min(1, ContainerWidth/GroupWidth) + "," + Math.min(1, ContainerHeight/GroupHeight) + ")");
   }  
   else{
    $("g#" + GroupId).attr("transform", "scale(1,1)");
   }
  }
                 
 this.show = 
  function (Editor){
   this.clear(Editor); 
   if(Editor.ifware.BlockDataSet){
    $("#" + ContainerId).svg({onLoad: function(svg){drawControl(svg, Editor);}});
   }

   TOOLinitialize();
  } 
}

var ETHscrollerControl = new ETHscroller("EDTthumbnailScroll");
