// Intellisense extension for a CodeMirror editor.
// Requires jquery and annotation.js.
// Also the stylesheet (intellisense.css) uses images from /js/treeview.
// Simply call ITSinitIntellisense() in the CodeMirror editor's initCallback, and you'll be good to go!
//
// TODO - it may be better to pass a "status change" callback function,
// instead of the specific ErrorPanel element.
//
// Author: Kevin Senn
function ITSinitIntellisense(Editor, Popup, ErrorPanel){
   function ITSjsEscape(Value){ return Value.replace(/\"/g, '\\"').replace(/\'/g, "\\'"); }

   function ITSdisplayIntellisenseError(ErrorMessage){ ErrorPanel.innerHTML = ErrorMessage; }

   function ITSonOptionClick(OptionText, OriginalExpression, LineHandle, CursorPos, HasMoreOptions){
      Popup.style.display = 'none';
      var OriginalLineContent = Editor.lineContent(LineHandle);
      var NewLineContent = OriginalLineContent.substring(0, CursorPos - OriginalExpression.length);
      NewLineContent += OptionText;
      NewLineContent += OriginalLineContent.substring(CursorPos);
      Editor.setLineContent(LineHandle, NewLineContent);
      Editor.selectLines(LineHandle, CursorPos + (OptionText.length - OriginalExpression.length));
      Editor.focus();
      if (HasMoreOptions){ ITSrequestIntellisense(OptionText); }
   }

   function ITSgetHighlightedOption(){ return $(Popup).find('table tr.hover').get(0); }

   function ITSscrollToSelectedOption(){
      var Row = ITSgetHighlightedOption();
      var RowBeg = Row.offsetTop - 2;
      var RowEnd = Row.offsetTop + Row.offsetHeight + 2 + 20; // 20 accounts for a possible horizontal scrollbar in the popup.
      var ScrBeg = Popup.scrollTop;
      var ScrEnd = Popup.scrollTop + Popup.offsetHeight;
      var LimBeg = 0;
      var LimEnd = Popup.scrollHeight - Popup.offsetHeight + 20; // 20 accounts for a possible horizontal scrollbar in the popup.
      if (RowEnd > ScrEnd) Popup.scrollTop = Math.max(LimBeg, Math.min(LimEnd, RowEnd-Popup.offsetHeight)) + 4;
      if (RowBeg < ScrBeg) Popup.scrollTop = Math.max(LimBeg, Math.min(LimEnd, RowBeg));
   }

   function ITSselectOption(Row){ $(Row).addClass('hover'); }

   function ITSselectFirstOption(){
      var CurrentOption = ITSgetHighlightedOption();
      if (CurrentOption){ ITSdeselectOption(CurrentOption); }
      ITSselectOption($(Popup).find('table tr').get(0));
   }

   function ITSselectLastOption(){
      var CurrentOption = ITSgetHighlightedOption();
      if (CurrentOption) { ITSdeselectOption(CurrentOption); }
      var Options = $(Popup).find('table tr').get();
      ITSselectOption(Options[Options.length-1]);
   }

   function ITSdeselectOption(Row){ $(Row).removeClass('hover'); }

   function ITSoptionRowOnMouseOver() {
      ITSdeselectOption(ITSgetHighlightedOption());
      ITSselectOption(this);
   }

   function ITSoptionRowOnMouseOut(){ ITSdeselectOption(this); }

   function ITSgenerateOption(ProtocolType, Option, OriginalExpression, LineHandle, CursorPos){
      var OptionRow = document.createElement('tr');
      OptionRow.onmouseover = ITSoptionRowOnMouseOver;
      OptionRow.onmouseout  = ITSoptionRowOnMouseOut;
      OptionRow.onclick     = function() {
         ITSonOptionClick(Option.FullText, OriginalExpression, LineHandle, CursorPos, !Option.IsSimple);
      }
      var OptionCol = document.createElement('td');
      OptionCol.className = 'nt_' + ProtocolType + '_' + Option.NodeType;
      if (Option.IsRepeatNode) {
         OptionCol.className += ' r';
      }
      OptionCol.innerHTML = Option.DisplayText;
      OptionRow.appendChild(OptionCol);
      return OptionRow;
   }

   function ITSonEditorKeyDown(e){
      var KeyCode = (window.event ? event.keyCode : e.keyCode);
      switch( KeyCode ){
      case 32:  // Space
      case 13:  // Enter
         var Option = ITSgetHighlightedOption();
         Option.onclick();
         return;
      case 38:  // Up-arrow
         var CurrentOption = ITSgetHighlightedOption();
         var PreviousOption = $(CurrentOption).prev().get(0);
         if (CurrentOption && PreviousOption){
            ITSdeselectOption(CurrentOption);
            ITSselectOption(PreviousOption);
         } else {
            ITSselectFirstOption();
         }
         ITSscrollToSelectedOption();
         return;
      case 40:  // Down-arrow
         var CurrentOption = ITSgetHighlightedOption();
         var NextOption = $(CurrentOption).next().get(0);
         if (CurrentOption && NextOption) {
            ITSdeselectOption(CurrentOption);
            ITSselectOption(NextOption);
         } else if (!CurrentOption) {
            ITSselectFirstOption();
         }
         ITSscrollToSelectedOption();
         return;
      case 36:  // Home
         ITSselectFirstOption();
         ITSscrollToSelectedOption();
         return;
      case 35:  // End
         ITSselectLastOption();
         ITSscrollToSelectedOption();
         return;
      }
   }

   var ITSonCharacterTypedTimeoutId = null;
   function ITSgrabKeyFilter(KeyCode){
      if (Popup.style.display == ''){ // if Popup is showing
         switch( KeyCode ){
         case 32:  // Space
         case 13:  // Enter
            var Option = ITSgetHighlightedOption();
            if (Option){ return true; }
            break;
         case 36:  // Home
         case 35:  // End
         case 38:  // Up-arrow
         case 40:  // Down-arrow
            return true;
         }
      }
      // Characters that won't modify the existing expression.
      if ( KeyCode == 0 || // Mysterious keycode that sometimes gets sent after a real one
           KeyCode == 8 || // Backspace
           (KeyCode >= 13 && KeyCode <= 46) || // Enter to Delete
           (KeyCode >= 91 && KeyCode <= 93) || // Left window key to Select
           (KeyCode >= 112 && KeyCode <= 145) ){ // Function keys to Scroll lock
         return false;
      }
      // We use the "setTimeout" trick to allow the character to actually be added to the code.
      if ($.browser.msie) {
         // Put a small delay for IE to catch up.
         if (ITSonCharacterTypedTimeoutId) { clearTimeout(ITSonCharacterTypedTimeoutId); }
         ITSonCharacterTypedTimeoutId = setTimeout(function(){ ITSrequestIntellisense(); }, 110);
      } else {
         if (ITSonCharacterTypedTimeoutId) { clearTimeout(ITSonCharacterTypedTimeoutId); }
         ITSonCharacterTypedTimeoutId = setTimeout(function(){ ITSrequestIntellisense(); }, 0);
      }
      return false;
   }

   function ITScalculatePopupPosition(LineNumber, CursorPos, FullLine, OriginalExpressionLength, TrimmedExpressionLength){
      var CodeIframe = $(Editor.frame);
      var CodeIframeOffset = CodeIframe.offset();
      // Get offset of iframe relative to popup's parent.
      var PopupParentNodeOffset = $(Popup).parent().offset();
      var RelativeIframeLeft = CodeIframeOffset.left - PopupParentNodeOffset.left;
      var RelativeIframeTop  = CodeIframeOffset.top  - PopupParentNodeOffset.top;

      var EditBox = CodeIframe.contents().find('body').get(0);
      var TesterSpan = EditBox.appendChild(document.createElement('span'));

      try{
         TesterSpan.style.whiteSpace = 'pre';
         var CharsToTrim = OriginalExpressionLength - TrimmedExpressionLength;
         FullLine = FullLine.substring(0, CursorPos - CharsToTrim);
         TesterSpan.innerHTML = FullLine;
         var NewLeft = TesterSpan.offsetWidth + RelativeIframeLeft  - EditBox.scrollLeft + 5; // 5 is width of the margin

         var LineTops = NISgetLines(NISgetEditorDiv(Editor));
         var NewTop = LineTops[LineNumber+1];
         if (LineTops.length-1 <= LineNumber){
            NewTop = LineTops[LineNumber] + 17; // 17 is the line height
         }
         NewTop += RelativeIframeTop - EditBox.scrollTop;
         
         // Calculate bottom to use, if we place the popup above the current line, rather than under.
         var NewBottom = LineTops[LineNumber] + RelativeIframeTop - EditBox.scrollTop;
         EditBox.removeChild(TesterSpan);
         return { left:NewLeft, top:NewTop, bottom:NewBottom,
                  absLeft:NewLeft + PopupParentNodeOffset.left,
                  absTop: NewTop + PopupParentNodeOffset.top };
      } catch (e){
         EditBox.removeChild(TesterSpan);
         throw e;
      }
   }

   function ITSintellisenseOptionsHandler(Response, TextStatus, RequestObject){
      m_CurrentAjaxRequest = null;
      ErrorPanel.innerHTML = '';
      if (Response.ErrorMessage){
         ITSdisplayIntellisenseError(Response.ErrorMessage);
         return;
      }
      var Options = Response.Options;
      if (Options && Options.length > 0){
         var CursorPos = Editor.cursorPosition().character;
         var Line = Editor.cursorLine();
         var LineNumber = Editor.lineNumber(Line);
         var LineContent = Editor.lineContent(Line);

         if (LineNumber == Response.LineNumber && CursorPos == Response.CursorPosition){
            var OptionsTable = document.createElement('table');
            for (var i = 0; i < Options.length; i++){
               OptionsTable.appendChild(ITSgenerateOption
                  (Response.ProtocolType, Options[i], Response.FullExpression, Line, CursorPos)
               );
            }
            if (Response.OptionsTruncated){
               var WarningRow = document.createElement('tr');
               var WarningCol = document.createElement('td');
               WarningCol.className = 'intellisenseOptionsTruncatedWarning';
               WarningCol.innerHTML = 'Not all results could be displayed.';
               WarningRow.appendChild(WarningCol);
               OptionsTable.appendChild(WarningRow);
            }

            Popup.cursorPos = CursorPos;
            Popup.lineNo = LineNumber;
            $(Popup).empty();
            Popup.appendChild(OptionsTable);
            var PopupPosition = ITScalculatePopupPosition(LineNumber, CursorPos, LineContent, Response.FullExpression.length, Response.TrimmedExpression.length);

            Popup.style.left = PopupPosition.left + 'px';
            Popup.style.top  = PopupPosition.top  + 'px';
            Popup.style.height = '';
            Popup.style.width = '';
            if ($(Popup).outerHeight() > 150){
               Popup.style.height = '150px';
               Popup.style.width = ($(Popup).innerWidth() + 16) + 'px'; // make up space lost from scrollbar
            }
            // Limit the width of the popup so that it doesn't run off the window.
            var WindowWidth = WINgetWindowWidth();
            if (PopupPosition.absLeft + $(Popup).outerWidth() > WindowWidth){
               Popup.style.width = (WindowWidth - PopupPosition.absLeft - 16) + 'px'; // the 16 just gives us a bit of padding
            }
            // If the bottom of the popup goes off the window, put it above the current line instead.
            var PopupOuterHeight = $(Popup).outerHeight();
            if (PopupPosition.absTop + PopupOuterHeight > WINgetWindowHeight()){
               Popup.style.top = (PopupPosition.bottom - PopupOuterHeight) + 'px';
            }
            Popup.style.display = '';
         }
      }
   }

   function ITSrequestIntellisense(AdditionalRequestExpression) {
      var CursorPos = Editor.cursorPosition().character;
      var LineNumber = Editor.lineNumber(Editor.cursorLine());

      Popup.style.display = 'none';
      ErrorPanel.innerHTML = '<img src="/js/mapper/images/spinner.gif" />';
      if (m_CurrentAjaxRequest) {
         // Abort the request (client-side only).
         m_CurrentAjaxRequest.onreadystatechange = function() {};
         m_CurrentAjaxRequest.abort();
      }
      var Vars = { 'LineNumber' : LineNumber, 'CursorPosition' : CursorPos }
      if (AdditionalRequestExpression){
         Vars.AdditionalRequestExpression = AdditionalRequestExpression;
      }
      // using a common function to pack in Lua function parameter information
      NISprepareLuaPostInfo(Editor, Vars, true); // Need lua code

      m_CurrentAjaxRequest = Ajax.post( '/get_intellisense_options', Vars, ITSintellisenseOptionsHandler, 'json');
   }

   function ITScursorActivity(CursorSpan) {
      var CursorPos = Editor.cursorPosition().character;
      var LineNumber = Editor.lineNumber(Editor.cursorLine());
      if (Popup.style.display == '' && (Popup.cursorPos != CursorPos || Popup.lineNo != LineNumber)) { Popup.style.display = 'none'; }
   }

   // Setup cursor activity callback
   if (Editor.options.cursorActivity) {
      var OriginalCursorActivityCallback = Editor.options.cursorActivity;
      Editor.options.cursorActivity = function(CursorSpan) { ITScursorActivity(CursorSpan); OriginalCursorActivityCallback(CursorSpan); }
   } else {
      Editor.options.cursorActivity = ITScursorActivity;
   }
   
   // Setup key event callbacks
   Editor.grabKeys(ITSonEditorKeyDown, ITSgrabKeyFilter);
   
   function ITSonEditorScroll(){ Popup.style.display = 'none'; }
   
   if (Editor.win.addEventListener) {
      Editor.win.addEventListener('scroll', ITSonEditorScroll, false);
   } else if (Editor.win.document.attachEvent) {
      Editor.win.document.attachEvent('onscroll', ITSonEditorScroll);
   }
   var m_CurrentAjaxRequest = null;
}
