//
// Treeview for a NOD tree.
// Requires /js/treeview/treeview_2_0.js and jquery.
//
// Author: Kevin Senn
//

// The presence of ExpansionTree indicates that this node should be
// expanded.  The tree then indicates which (if any) children should
// be expanded.
function NTRcreateTreeNode(DataNode, ExpansionTree)
{
   var NodeClass = 'nt_' + DataNode.pt + '_' + DataNode.nt;
   var LiClass = '';
   if (DataNode.e)
   {
      NodeClass += ' e';
      LiClass = 'e';
   }
   if (DataNode.r)
   {
      NodeClass += ' r';
   }
   
   var IsExpanded = ExpansionTree && DataNode.c.length;

   var NewNode = TRVcreateNode(DataNode, LiClass, NodeClass, DataNode.c.length,
                               IsExpanded, IsExpanded ? DataNode.ev : DataNode.cv, NTRonNodeToggled);
   
   if (ExpansionTree){
      var ChildContainer = TRVgetChildContainer(NewNode);
      for (var i = 0; i < DataNode.c.length; i++){
         ChildContainer.appendChild(NTRcreateTreeNode(DataNode.c[i], ExpansionTree[i]));
      }
   }
   
   return NewNode;
}

function NTRonNodeToggled(DataNode, NodeTextSpan, NodeChildrenContainer, Expanded)
{
   if (Expanded)
   {
      var CountOfChild = DataNode.c.length;
      for (var ChildIndex = 0; ChildIndex < CountOfChild; ++ChildIndex)
      {
         NodeChildrenContainer.appendChild(NTRcreateTreeNode(DataNode.c[ChildIndex]))
      }

      NodeTextSpan.innerHTML = DataNode.ev;
   }
   else
   {
      NodeTextSpan.innerHTML = DataNode.cv;
   }
}

function NTRpopulateExpansionTree(Children, ExpansionTree){
   Children = $(Children); // ensure it's a jQuery object
   Children.children('li.collapsable').each(function(){
      var ThisIndex = $(this).prevAll().length;
      if (!ExpansionTree[ThisIndex]){
         ExpansionTree[ThisIndex] = {};
      }
      NTRpopulateExpansionTree(TRVgetChildContainer(this), ExpansionTree[ThisIndex]);
   });
}


// PUBLIC INTERFACE FUNCTIONS BELOW

// Generate an "expansion tree" that represents the current state of the specified treeview.
// Examples:
// A tree with the root node collapsed will have an expansion tree of: {}
// A tree with only the root node expanded will have an expansion tree of: {0:{}}
// A tree with the third child of the root node (as well as the root node) expanded
// will have an expansion tree of: {0:{ 2:{} }}
//
function NTRgetExpansionTree(Treeview){
   var ExpansionTree = {};
   NTRpopulateExpansionTree(Treeview, ExpansionTree);
   return ExpansionTree;
}

// ExpansionTree is a javascript object which specifies which nodes
// should be expanded when the node is created.  See the description of
// NTRgetExpansionTree() above.
// If ExpansionTree is omitted, the root node (and only the root node) will be expanded.
//
function NTRinitTreeview(TreeRoot, Container, ExpansionTree){
   Container.innerHTML = '';
   if (!ExpansionTree){
      ExpansionTree = {0:{}};
   }
   Container.appendChild(NTRcreateTreeNode(TreeRoot, ExpansionTree[0]));
}
