// The page.js file should be kept very small.  Please note there is a deliberate
// structure here to keep it to just the map of pages to their handler functions
// and the associated utility functions.  
//
// In general the pattern is that each screen or group of screens should have their
// own associated implemetation file to make it easier to navigate the file structure
// and know where to find stuff.

// See DBD/web_docs/mapper_config.js for Iguana and node_test/webdocs/page_lookup.js for node_test
// for the definition of MPRapp.pageLookup

function PAGsetupCommonMapCalls(Lookup){
 Lookup['OpenEditor']      = EDTopenEditor;
 Lookup['OpenLog']         = LOGopenLog;
 Lookup['SampleData']      = SPDview;
 Lookup['EditSampleData']  = SPDEedit;
 Lookup['BulkTestResults'] = BTRview;
} 

// returns value of a href in <a href="...">  with the right escaping to make a link to a page
// Link = "<a href='" + PAGhref("OpenEditor", { Script : C.script, Data : C.data} ) + "'>MyLink</a>";
function PAGhref(Page, Params){
 var Anchor = "#Page=" + Page;
 if (Params){
  for (i in Params){
   Anchor += "&" + i + "=" + PAGescapeQuotes(encodeURIComponent(Params[i]));
  }
 }
 return Anchor;
}

// returns value of a href in <a href="...">  with the right escaping to make a link to a page
// Link = PAGlink("OpenEditor", { Script : C.script, Data : C.data} ) + "'>MyLink</a>";
function PAGlink(Page, Params, Text){
 if(Params.Page != "undefined"){
  delete Params.Page;
 }
 return "<a href='" + PAGhref(Page, Params) + "'>" + Text + "</a>"
}

// return string with single quotes escaped
function PAGescapeQuotes(Input) {
 return Input.replace(/'/g, "&#39;").replace(/"/, "&quot;");
}

// NOTE: This can only be called on an encoded string.  With decoded strings, pluses will be incorrectly converted 
// to the general encoding.
function PAGconvertSpaceEncoding(EncodedInput){
 return EncodedInput.replace(/\+/g, "%20");
}
 
function PAGhtmlEscape(Text){
 return Text.replace(/;/g, '&#59;')
            .replace(/&(?!#59;)/g, '&amp;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;')
            .replace(/"/g, '&quot;');
}

function PAGgetField(Params, Name){
 if (typeof (Params[Name]) === 'undefined'){
  throw new Error('Need to have ' + Name + ' defined.'); 
 }
 return Params[Name];
}

function PAGgetOptionalField(Params, Name){
 if (typeof (Params[Name]) === 'undefined'){
  return "";
 }
 return Params[Name];
}

function PAGparseAnchorString(AnchorString){
 var Params = {};
 var Vars = AnchorString.split('&');
 for (var i=0; i < Vars.length; i++){
  var Parts = Vars[i].split('=');
  if(undefined != Parts[1]){
   Parts[1] = PAGconvertSpaceEncoding(Parts[1]);
  }
  Params[Parts[0]] = decodeURIComponent(Parts[1]);
 }
 return Params;
}

function PAGcallPage(AnchorString){
 // Reset the page's onresize handler (see #16696)
 window.onresize = null;
 console.log(AnchorString);
 var AnchorParts = AnchorString.split('#');
 console.log(AnchorParts);
 if (AnchorParts.length < 2){
  MPRapp.defaultRequest("No page specified.");
  return;
 }
 var JustAnchor = AnchorParts.pop();
 if (JustAnchor == 'error') return;
 Params = PAGparseAnchorString(JustAnchor);

 if("Explorer" == BrowserDetect.browser && 9 > BrowserDetect.version){
  MPRapp.renderError("Unfortunately, the Translator does not support Internet Explorer " + BrowserDetect.version + ". Please use a standards compliant browser.", Params);
  return;
 }

 if (typeof(Params.Page) === 'undefined' || "" == Params.Page){
  MPRapp.defaultRequest("No page specified.");
  return;
 }
 if (typeof(MPRapp.pageLookup[Params.Page]) === 'undefined'){
  MPRapp.renderError("Page " + Params.Page + " not found.", Params);
  return;
 }
 MPRapp.pageLookup[Params.Page](Params);
} 

var PAGlocation = '';

$(document).ready(function(){
 // document.documentMode considered because IE 8 in compatibility mode reports true for this event.  
 if('onhashchange' in window && ( document.documentMode === undefined || document.documentMode > 7 )){
  window.onhashchange = PAGonHashChange;
 }
 else{
  // We're going to polling location.href.
  setInterval(PAGcheckForHashChange, 100);    
 } 
});

function PAGcheckForHashChange(){
 if(PAGlocation != location.href){
  PAGlocation = location.href;
  console.log("Hash change= ", PAGlocation);
  PAGcallPage(location.href);
 } 
}

function PAGonHashChange(){
 console.log("Hash change= ", location.href);
 PAGcallPage(location.href);
}

