/*
 * Creates and returns a DOM element for a tree node.  The element should be put inside
 * a <ul> tag with class "ifwareTreeview".
 * DataNode is any generic object, that will be passed into the onNodeToggled and onNodeClicked event handlers.
 * LiClass is a class that will be applied to the entire tree node.  For example, if the class had the property
 *         display:none, then the node will not be shown in the tree (and there will be no empty space).
 * NodeClass is a class that will only be applied to the node's icon and text.
 * IsExpandable specifies whether or not the node is expandable/collapsable.
 * IsExpanded specifies whether or not the node should start off expanded.
 */
function TRVcreateNode(DataNode, LiClass, NodeClass, IsExpandable, IsExpanded, NodeText, onNodeToggled, onNodeClicked)
{
   var NodeLi = document.createElement('li');
   
   if (IsExpandable && IsExpanded){
      NodeLi.className = 'collapsable';
   }
   else if (IsExpandable && !IsExpanded){
      NodeLi.className = 'expandable';
   }
   else{
      NodeLi.className = 'leaf';
   }
   
   if (LiClass) NodeLi.className += ' ' + LiClass;
   
   NodeLi.innerHTML =
      '<div>\
          <span class="treeIcon"></span>\
          <span class="nodeText ' + NodeClass + '">' + NodeText + '</span>\
       </div>\
       <ul' + (IsExpanded ? '' : ' style="display:none;"') + '></ul>';
   
   $(NodeLi).children('div').click(function(EventObject)
   {
      var NodeSpan     = $(NodeLi).find('span.nodeText:first').get(0);
      var NodeChildren = $(NodeLi).find('ul:first').get(0);
      
      NodeChildren.innerHTML = '';
      
      if ($(NodeLi).hasClass('expandable')){
         $(NodeLi).removeClass('expandable');
         $(NodeLi).addClass('collapsable');
      
         NodeChildren.style.display = '';
         
         onNodeToggled(DataNode, NodeSpan, NodeChildren, true);
      }
      else if ($(NodeLi).hasClass('collapsable')){
         $(NodeLi).removeClass('collapsable');
         $(NodeLi).addClass('expandable');
         
         NodeChildren.style.display = 'none';
         
         onNodeToggled(DataNode, NodeSpan, NodeChildren, false);
      }
      
      if (onNodeClicked) onNodeClicked(DataNode, NodeSpan, NodeChildren);

      EventObject.stopPropagation();
   });
   
   if (IsExpandable || onNodeClicked){
      $(NodeLi).find('span.nodeText:first')
      .mouseover(function(){ $(this).addClass('hover')    })
      .mouseout( function(){ $(this).removeClass('hover') });
   }
   
   return NodeLi;
}

function TRVgetChildContainer(NodeLi){
 return $(NodeLi).find('ul:first').get(0);
}
